//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef SETTINGSH
#define SETTINGSH

#include <vector>
#include <map>
#include <string>
#include <fstream>

#include "common.h"


// The Settings class will contains all the global settings for the program
// We want Settings to be a singleton class, that can be instanciate 
// only once and cannot be duplicate.
// WARNING : this singleton feature is not thread safe. It is better to instanciate the settings class before running any threads.


class Settings 
{
 public:

  // The public constructor: only the first call will create an instance
  static Settings *GetInstance () 
  {
    if (!Instance) 
      Instance = new Settings;
    return Instance;
  };
  
  void ParseCommandLine(int argc, char **argv,
			std::string format,
			std::vector<std::string>& args,
			std::vector<std::pair<std::string,std::string> >& opt);
  bool IsInteractive(); 
  void ReadConfig(std::string name = "");

  std::vector<std::map<std::string, std::string> >& GetNeighborhoodConfig() 
    { return neighborhood;};
  std::vector<std::map<std::string, std::string> >& GetFieldConfig() 
    { return field;};
  const std::vector<std::map<std::string, std::string> >& GetFields() const {return field;};  

  const std::string& GetCurrentFilename()const{
    return this->currentFilename;
  };

  bool interactiveMode;
  std::string configFileName;
  std::string currentFilename;

 private:
  static Settings *Instance; // The only existing Instance
  
  // The private constructor
  Settings () {
    interactiveMode = false;
  }
  ~Settings(){};  
  // The two empty methods make it impossible to duplicate the class
  Settings (const Settings &);
  void operator= (const Settings &);
  
  // list of associative list <"description", value>
  std::vector<std::map<std::string, std::string> > neighborhood;
  std::vector<std::map<std::string, std::string> > field;

 
};

// Instanciate the settings class to store program parameters
// As we want only one shared instance of settings it is better to do it here.
static Settings * settings = Settings::GetInstance();

#endif
