//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef BYPRODUCTH
#define BYPRODUCTH

#include <vector>

class ByProduct 
{
 public :
  
  ByProduct(std::vector<double>& values);
  ~ByProduct();

  void Compute(std::string method, 
	       std::vector<std::vector<double>*>& items,
	       const std::map<std::string,std::string>& prop);

 private:
  std::vector<double>& field;

  void Resize(unsigned int sourceSize);
  void ComputeMean(std::vector<std::vector<double>*>& items);
  void ComputeMax(std::vector<std::vector<double>*>& items);
  void ComputeMaxNormalized(std::vector<std::vector<double>*>& items);
  void ComputeWeightedSum(std::vector<std::vector<double>*>& items,
			  std::vector<double>& weights);
  void ComputeWeightedSumNormalized(std::vector<std::vector<double>*>& items,
				    std::vector<double>& weights);
  void ComputeWeightedSumWeightNormalised(std::vector<std::vector<double>*>& items,
					  std::vector<double>& weights);
  void ComputeNormalize(std::vector<double>& items);
  void ComputeProduct(std::vector<double>& item1,std::vector<double>& item2);
  void ComputeShapeIndex(std::vector<double>& item1,std::vector<double>& item2);
  void ComputeCurvedness(std::vector<double>& item1,std::vector<double>& item2);
  void ComputeLogCurvedness(std::vector<double>& item1,std::vector<double>& item2);
  void ComputeWillmore(std::vector<double>& item1,std::vector<double>& item2);
  void ComputeDibeklioglu(std::vector<double>& item1,std::vector<double>& item2);
  void ComputeDiff(std::vector<double>& item1, 
		   std::vector<double>& item2);


};

#endif
