//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include"curvature.h"
#include"../geom.h"


//----------------------------------------------------------------------------
Curvature::Curvature(const Neighborhood* n)
  : neighborhood(n)
{
  this->Initialise();
}

//----------------------------------------------------------------------------
Curvature::~Curvature()
{
  ks1.clear(); 
  ks2.clear(); 
  k1 .clear(); 
  k2 .clear();
  for (int i=0; i<(int)t1.size(); i++){
    delete [] t1[i];
    delete [] t2[i];
  }
  t1 .clear();
  t2 .clear(); 

  for (int i=0; i<(int)normals.size(); i++){
    delete [] normals[i];
  }
}



void Curvature::Initialise(){
  this->mesh = this->neighborhood->GetMesh();
  
  int npoint = this->mesh->Points().size();
  
  ks1 = std::vector<double>(npoint, 0.0);
  ks2 = std::vector<double>(npoint, 0.0);
  k1  = std::vector<double>(npoint, 0.0);
  k2  = std::vector<double>(npoint, 0.0);

  t1  = std::vector<double*>(npoint);
  t2  = std::vector<double*>(npoint);
  for(int i=0; i<npoint; i++){
    t1[i] = new double[3];
    t2[i] = new double[3];
  }
}


int Curvature::GetSize(){
  return this->k1.size();
}


void Curvature::PrintInFile(const std::string filename)
{
  FILE* fout = fopen(filename.c_str(), "w");
  if (!fout){
    fprintf(stderr,"error opening file\n");
  }
  fprintf(fout, "#description id k1 k2 ks1 ks2 t1 t2\n");
  for (int i=0; i<(int)k1.size(); i++){
    fprintf(fout, "%lf ", k1[i]);
    //fprintf(fout, "%d %lf %lf %lf %lf %lf,%lf,%lf %lf,%lf,%lf \n",
    //	    i, k1[i], k2[i], ks1[i], ks2[i], 
    //	    t1[i][0], t1[i][1], t1[i][2], t2[i][0], t2[i][1], t2[i][2]);
  }
  fclose(fout);    
}


//void Curvature::ComputeByProduct(){
//  unsigned int pointNb = this->k1.size();
//  this->gauss.resize(pointNb);
//  this->mean.resize(pointNb);
//  for (unsigned int i=0; i<pointNb; i++){
//    this->gauss[i] = this->k1[i] * this->k2[i];
//    this->mean[i] = this->k1[i] + this->k2[i];
//  }
//}



//void Curvature::ComputePougetDirections(){
//
//  int npoint = (int)this->mesh->Points().size();  
//  for(int i =0; i<npoint; i++){   
//    if( ! Geom<double>::IsDirect_3(this->t1[i], this->t2[i], this->normals[i])){
//      Geom<double>::Switch_3(this->t1[i], this->t2[i]);
//      Geom<double>::Switch_1(&this->k1[i], &this->k2[i]);
//      Geom<double>::Switch_1(&this->ks1[i], &this->ks2[i]);
//    }
//  }
//}
