//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CURVATUREH
#define CURVATUREH

#include <vector>
#include <string>
#include <stdio.h>
#include "../mesh.h"
#include "../neighborhood.h"

class Curvature 
{
 public:
  Curvature(const Neighborhood* n); 
  virtual ~Curvature();

  void Initialise();  
  virtual void Compute(const std::vector<int>* subset=NULL) = 0;

  const std::vector<double*>& GetNormals() const {return normals;};
  const std::vector<double>& Getk1() const {return k1;};  
  const std::vector<double>& Getk2() const {return k2;};
  const std::vector<double*>& Getd1() const {return t1;};
  const std::vector<double*>& Getd2() const {return t2;};
  const std::vector<double>& Getb0() const {return ks1;};
  const std::vector<double>& Getb3() const {return ks2;};
  //const std::vector<double>& GetGaussCurvature() const {return gauss;};
  //const std::vector<double>& GetMeanCurvature() const {return mean;};

  const Mesh * GetMesh() const{return mesh;};
  int GetSize();
  void PrintInFile(const std::string filename);
    
  //void ComputeByProduct();

  //void ComputePougetDirections();
  void SetMethodString(std::string& m){
    method = m;
  };
  
 protected:
  std::string method;
  const Mesh * mesh;
  const Neighborhood * neighborhood;

  // Out
  // First derivative of principal curvature
  std::vector<double> ks1; 
  std::vector<double> ks2;    
  // Principal curvature
  std::vector<double> k1;
  std::vector<double> k2;
  // principal direction
  std::vector<double *> t1;
  std::vector<double *> t2;
  // by product
  //std::vector<double> gauss;
  //std::vector<double> mean;
  
  std::vector<double*> normals;

 private:
  
};



#endif
