//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "stopwatch.h"
#include "distanceToLocalPlane.h"
#include "../geom.h"

DistanceToLocalPlane::DistanceToLocalPlane(const Neighborhood * n,
					   const std::vector<Histogram*>* normals,
					   const std::vector<int>* subset)
{
  this->neighborhood = n;
  this->normals = normals;
  Stopwatch::Start("distanceToLocalPlane");
  this->Compute(subset);
  Stopwatch::Stop("distanceToLocalPlane");

}

DistanceToLocalPlane::~DistanceToLocalPlane()
{
  
}

void DistanceToLocalPlane::Compute(const std::vector<int>* subset)
{  
  const Mesh * mesh = this->neighborhood->GetMesh();
  unsigned int npoint = mesh->Points().size();
  this->distanceToLocalPlane.resize(npoint);

  std::vector<int> selection ;
  if (subset==NULL){
    for(unsigned int i=0; i<npoint; i++){
      selection.push_back(i);
    }
  }
  else{
    selection = (*subset);
  }  

  std::vector<int>::iterator
    itb = selection.begin(),
    ite = selection.end();
  for(; itb!= ite; itb++){
    unsigned int i=(*itb);
    distanceToLocalPlane[i] = 0.0;
    double O[3];
    O[0] = O[1] = O[2] = 0.0;

    std::vector<int>::const_iterator 
      itb = this->neighborhood->GetNeighborhoodPoints()[i].begin(), 
      ite = this->neighborhood->GetNeighborhoodPoints()[i].end();
    for(; itb != ite; itb++){
      Geom<double>::Add_3(O,mesh->Points()[(*itb)]);
    }

    Geom<double>::Divide_3(O, (double)this->neighborhood->GetNeighborhoodPoints()[i].size());
    
    double * N = dynamic_cast<HVector*>((*this->normals)[i])->GetRawData();

    distanceToLocalPlane[i] = Geom<double>::DistanceToPlan_3(mesh->Points()[i],
							     O,N);

  }
  
}

