//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "stopwatch.h"
#include "volume.h"
#include "../geom.h"

Volume::Volume(const Neighborhood * n, const std::vector<int>* subset)
{
  this->neighborhood = n;
  Stopwatch::Start("volume");
  this->Compute(subset);
  Stopwatch::Stop("volume");

}

Volume::~Volume()
{
  
}

void Volume::Compute(const std::vector<int>* subset)
{  
  const Mesh * mesh = this->neighborhood->GetMesh();
  unsigned int i;
  unsigned int npoint = mesh->Points().size();
  double pt[3];
  double v1[3];
  double v2[3];
  double v3[3];
  
  this->volume.resize(npoint);

  std::vector<int> selection ;
  if (subset==NULL){
    for(i=0; i<npoint; i++){
      selection.push_back(i);
    }
  }
  else{
    selection = (*subset);
  }  

  std::vector<int>::iterator
    itb = selection.begin(),
    ite = selection.end();
  for(; itb!= ite; itb++){
    i = (*itb);

    volume[i] = 0.0;
    pt[0] = pt[1] = pt[2] = 0.0;

    std::vector<HalfEdge*>::const_iterator 
      itb = this->neighborhood->GetNeighborhoodBorder()[i].begin(), 
      ite = this->neighborhood->GetNeighborhoodBorder()[i].end();
    for(; itb != ite; itb++){
      Geom<double>::Add_3(pt,pt,mesh->Points()[(*itb)->vertex]);
    }

    Geom<double>::Divide_3(pt, (double)this->neighborhood->GetNeighborhoodBorder()[i].size());
    
    std::vector<int>::const_iterator 
      itfb = this->neighborhood->GetNeighborhoodFaces()[i].begin(), 
      itfe = this->neighborhood->GetNeighborhoodFaces()[i].end();
    for(; itfb != itfe; itfb++){
      double* p0 = mesh->Points()[mesh->Faces()[(*itfb)].vertex[0]]; 
      double* p1 = mesh->Points()[mesh->Faces()[(*itfb)].vertex[1]]; 
      double* p2 = mesh->Points()[mesh->Faces()[(*itfb)].vertex[2]]; 
      Geom<double>::Substract_3(v1, p0, p1);
      Geom<double>::Substract_3(v2, p0, p2);				
      Geom<double>::Substract_3(v3, p0, pt);
      volume[i] += Geom<double>::Det_3(v1,v2,v3);

    }
    volume[i] = volume[i]/6.0;//*(double)this->neighborhood->GetNeighborhoodFaces()[i].size());

    
    //double z = Geom<double>::Distance_3(mesh->Points()[i], pt);
    double r = this->neighborhood->GetDistance(); 
    //if (volume[i]>0){ 
    //   volume[i] += PI*(2*r*r*r/3 - r*r*z + z*z*z/3);
    //}
    //else{
    //   volume[i] += PI*(2*r*r*r/3 - r*r*z + z*z*z/3);
    //}
    double totalsphere = 4/3*PI*r*r*r;
    volume[i] = volume[i]/totalsphere; 

  }
  
}

