//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "stopwatch.h"
#include "willmore.h"
#include "../geom.h"

Willmore::Willmore(const Mesh * m, const std::vector<int>* subset){
  this->mesh = m;
  Stopwatch::Start("willmore");
  this->Compute(subset);
  Stopwatch::Stop("willmore");
}

Willmore::~Willmore()
{
  
}

void Willmore::Compute(const std::vector<int>* subset)
{  
  unsigned int i;
  unsigned int npoint = mesh->Points().size();
   
  this->willmore.resize(npoint);
   std::vector<int> selection ;
  if (subset==NULL){
    for(i=0; i<npoint; i++){
      selection.push_back(i);
    }
  }
  else{
    selection = (*subset);
  }  

  std::vector<int>::iterator
    itb = selection.begin(),
    ite = selection.end();
  for(; itb!= ite; itb++){
    i = (*itb);
    willmore[i] = -2*PI;
 
    std::list<HalfEdge>::const_iterator 
      itb = mesh->HEdges()[i].begin(), 
      ite = mesh->HEdges()[i].end();
    for(; itb != ite; itb++){
      if (! (*itb).IsBorderEdge()){
	double *I = mesh->Points()[(*itb).opposite->vertex]; //i
	double *J = mesh->Points()[(*itb).vertex];
	double *L = mesh->Points()[(*itb).next->vertex];
	double *K = mesh->Points()[(*itb).opposite->next->vertex];
	
	double A[3],B[3],C[3],D[3];
	Geom<double>::Substract_3(A,J,K);
	Geom<double>::Substract_3(B,L,J);
	Geom<double>::Substract_3(C,I,L);
	Geom<double>::Substract_3(D,K,I);
	Geom<double>::Normalize_3(A);
	Geom<double>::Normalize_3(B);
	Geom<double>::Normalize_3(C);
	Geom<double>::Normalize_3(D);
	
	double cosBeta = (Geom<double>::DotProduct_3(A,C)*Geom<double>::DotProduct_3(B,D)  
			  - Geom<double>::DotProduct_3(A,B)*Geom<double>::DotProduct_3(C,D)  
			  - Geom<double>::DotProduct_3(B,C)*Geom<double>::DotProduct_3(D,A));

	willmore[i] += acos(cosBeta);
      }
    }

  }

}

