//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef GEODESICH
#define GEODESICH

#include <list>
#include <vector>
#include "mesh.h"

enum GeodesicType {DIJKSTRA,PLAN};


class visit {
 public:
  visit(){
    radius = -1;
    previous = -1;
    dist = 0.0;
  };
  int radius ;
  int previous ;    
  double dist;
};


class Geodesic
{
 public :
  
  Geodesic(const Mesh* mesh, GeodesicType T);
  Geodesic(const Mesh* m, std::vector<double*>* n, GeodesicType T);
  ~Geodesic();

  const Mesh * GetMesh() const { return  mesh;};
  GeodesicType GetType() const { return  type;};

  void ComputeDistanceMatrix(std::vector<int> vertexIds, 
			     std::vector<std::vector<double> >& distanceMatrix);

  void ComputeDistanceMatrixDijkstra(std::vector<int> vertexIds, 
				     std::vector<std::vector<double> >& distanceMatrix);


  void ComputeDistanceMatrixDijkstraFast(std::vector<int>& vertexIds, std::vector<std::vector<double> >& distanceMatrix,std::vector<std::vector<std::list<int> > >& paths, bool computePath=true);

  
  void ComputePlanNormal(int a, int b, double *N);
  double DistanceCoarse(int A, int B);
  inline const HalfEdge* BestNextPoint(int a, double * A, double * B,  double * Q, 
				       double * PlanNormal, double * Dir,
				       const std::vector<int>& selected);


  void ComputeDistanceMatrixPlan(std::vector<int> vertexIds, std::vector<std::vector<double> >& distanceMatrix);

  void SetMaxDistance(double d){
    maxDistance = d;
  }
  
 private:
  const Mesh * mesh;
  GeodesicType type;
  std::vector<double*>* normals; 
  double maxDistance;
};

#endif
