//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef HISTOGRAMH
#define HISTOGRAMH

#include <vector>
#include <list>
#include <string>
#include <iostream>
#include <cmath>



// class to store either vectors or more complicated non-scalar structures
//cc should be merged to a meta class "Attributes"
class Histogram
{
 public :
  
  Histogram(){};
  virtual ~Histogram(){};
  int GetDegree(){ return degree;};
  
 protected:
  int degree;
};




//cc ugly but i dont have time
class HVector :public Histogram
{
 public :
  
  HVector(double * input, int size=3){
    degree = 1;
    data = new double[size];
    for (int i=0;i<size;i++){
      data[i] = input[i];
    }
  };
  ~HVector(){delete [] data;};
  
  double * GetRawData(){return data;};
 protected:
  double* data;
};



class HMatrix : public Histogram
{
 public :

  HMatrix(){
    degree = 2;
  };
  
  HMatrix(int x, int y, double value=0.0){
    degree = 2;
    this->SetSize(x,y,value);
  };
  ~HMatrix(){};

  void SetSize(int x, int y, double value=0.0){
    data.resize(x,std::vector<double>(y,value));
  }
 
  inline std::vector<double>& operator[](int i1){
    return data[i1];
  }
  std::vector<std::vector<double> >&  GetData(){return data;};

  void Normalize(double target=1.0){
    double sum = 0;
    for (unsigned int i=0; i<data.size(); i++){
      for (unsigned int j=0; j<data[i].size(); j++){
	sum += data[i][j];
      }
    }
    if (sum != 0.0){
      for (unsigned int i=0; i<data.size(); i++){
	for (unsigned int j=0; j<data[i].size(); j++){
	  data[i][j] =  data[i][j] / sum * target;
	}
      }
    }
  }


  double AbsDiffSum(HMatrix& hm2){
    double sum=0.0;
    for (unsigned int i=0; i<data.size(); i++){
      for (unsigned int j=0; j<data[i].size(); j++){
	sum += fabs(data[i][j] -hm2.GetData()[i][j]) ;
      }
    }
    return sum;
  }



   void Print(std::ostream& stream)const{
    for(unsigned int i = 0; i<data.size(); i++){
      if (i !=0){
	stream << ";" ;
      }
      for(unsigned int j = 0; j<data[i].size(); j++){
	if (j !=0){
	  stream << "," ;
	}

	stream << data[i][j] ;
      }
    }
  
  }



 protected:
  std::vector<std::vector<double> > data;
};



class PolarHistogram : public HMatrix{
 public :


  double AbsDiffSum(PolarHistogram& hm2,int offset){
    double sum=0.0;
    for (unsigned int i=0; i<data.size(); i++){
      for (unsigned int j=0; j<data[i].size(); j++){
	int OO = (j+offset)%data[i].size();
	sum += fabs(data[i][OO] -hm2.GetData()[i][j]) ;
      }
    }
    return sum;
  }

  // compare with
  double AbsDiffAxialSymetricSum(int offset){
    double sum=0.0;

    for (unsigned int i=0; i<data.size(); i++){
      unsigned int N = data[i].size();
      for (unsigned int j=0; j<N; j++){
	// symetry offset
	// offset = 0 , N= 8;
	// 0 -> (8-1-(0-0)%8+0) -> 7 
	// 1 -> (8-1-(1-0)%8+0) -> 6 
	// 2 -> (8-1-(2-0)%8+0) -> 5 
	// 3 -> (8-1-(3-0)%8+0) -> 4 
	// 4 -> (8-1-(4-0)%8+0) -> 3 
	// 5 -> (8-1-(5-0)%8+0) -> 2 
	// 6 -> (8-1-(6-0)%8+0) -> 1 
	// 7 -> (8-1-(7-0)%8+0) -> 0 
	// offset = 1 , N= 8;
	// 0 -> (8-1-(0-1)%8+1) -> (8-1-7+1) -> 1 
	// 1 -> (8-1-(1-1)%8+1) -> (8-1-0+1) -> 0 
	// 2 -> (8-1-(2-1)%8+1) -> (8-1-1+1) -> 7 
	// 3 -> (8-1-(3-1)%8+1) -> (8-1-2+1) -> 6 
	// 4 -> (8-1-(4-1)%8+1) -> (8-1-3+1) -> 5 
	// 5 -> (8-1-(5-1)%8+1) -> (8-1-4+1) -> 4 
	// 6 -> (8-1-(6-1)%8+1) -> (8-1-5+1) -> 3 
	// 7 -> (8-1-(7-1)%8+1) -> (8-1-6+1) -> 2 

	int OO = (N-1-(j-offset)%N+offset)%N;
	sum += fabs(data[i][OO] -data[i][j]) ;
      }
    }
    return sum;
  };

};


//std::ostream& operator<<(std::ostream& stream,HMatrix& hm){
//  hm.Print(stream);
//  return stream;
//}


#endif
