//
// Copyright (C) 2010  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef NEIGHBORHOODH
#define NEIGHBORHOODH

#include <list>
#include <vector>
#include "mesh.h"

enum DistanceType {LEVEL_DISTANCE, EUCLIDIAN_DISTANCE};

class Neighborhood
{
 public :
  
  Neighborhood(const Mesh* mesh, int maxlevel, const std::vector<int>*subset=NULL);
  Neighborhood(const Mesh* mesh, double maxdistance, const std::vector<int>*subset=NULL);
  ~Neighborhood();

  const std::vector< std::vector<int> >&       GetNeighborhoodFaces () const { 
    return  neighborhoodFaces ;};	    
  const std::vector< std::vector<int> >&       GetNeighborhoodPoints() const { 
    return  neighborhoodPoints;};	    
  const std::vector< std::vector<HalfEdge*> >& GetNeighborhoodBorder() const { 
    return  neighborhoodBorder;};

  const Mesh * GetMesh() const { return  mesh;};
  double GetDistance()const{return maxDistance;};
  double GetType()const{return distanceType;};

  std::vector<double> ComputeSum(const std::vector<double>& field) const; 
  std::vector<double> ComputeMean(const std::vector<double>& field) const; 
  void PrintStats();
  void Export(std::string str);
 private:
  const Mesh * mesh;

  void BuildLargeNeighborhood(const std::vector<int>*subset=NULL);

  // contains the list of object in the fix size neighborhood.
  // Warning : the points list do not contains the center point
  // it contains only the neighbors 
  std::vector< std::vector<int> > neighborhoodFaces;
  std::vector< std::vector<int> > neighborhoodPoints;
  // the borders contains only the halfedge in the periphery
  // of the neighborhood
  std::vector< std::vector<HalfEdge*> > neighborhoodBorder;

  inline bool IsDistanceOk(int i, HalfEdge * he, int* level);  
  inline bool IsDistanceOk(int i, int j, int*level=NULL);
  int distanceType; 
  int maxLevel;  
  double maxDistance; 

};

#endif
